@@ -23,9 +23,11 @@ class Logger(object):
                  loglev=None, #all available function or levels for logging
                  func_fields=None, # function which will be logged in every log
                  buffering=0,
-                 related_folder = None
+                 related_folder = None,
+                 splitting_blocks = False #block can be splited in couple files
                  ):
-        if related_folder:
+        self.splitting_blocks = splitting_blocks
+        if related_folder is not None:
             related_folder = os.path.abspath(related_folder)
             if not related_folder.endswith('/'):
                 related_folder += '/'
@@ -39,23 +41,25 @@ class Logger(object):
         self.action_log_file = None
         self.action_log_fh = None
         
-        loglev = loglev or  [
-                'crt',#critical error
-                'err',#error
-                'imp',#important information
-                'inf',#information
-                'log',#base log
-                'trc',#trace some data
-                'dbg'#debug information
-                ]
-        level = level or loglev
+        if loglev is None:
+            loglev =   [
+                    'crt',#critical error
+                    'err',#error
+                    'imp',#important information
+                    'inf',#information
+                    'log',#base log
+                    'trc',#trace some data
+                    'dbg'#debug information
+                    ]
+        if level is None:
+            level = loglev
         if isinstance(level,int):
             level = loglev[:level]
 
 
 
-
-        func_fields = func_fields or ['stacksize','fname','filename','lineno']
+        if func_fields is None:
+            func_fields =  ['stacksize','fname','filename','lineno']
         self.aindex_stack = []
         self.aindex_empty_level = None
         
@@ -74,8 +78,8 @@ class Logger(object):
         
         self.func_fields = self.arr_lambda_by_name(func_fields, every)
 
-    def __call__(self,*args,**kwargs):
-        return self.def_log_call(*args,**kwargs)
+    def __call__(self,a,s=0):
+        return self.def_log_call(a,s+1)
 
     def get_trio_log(self,logname):
         return getattr(self,logname),getattr(self,'a_'+logname),getattr(self,'c_'+logname)
@@ -91,7 +95,11 @@ class Logger(object):
             '2_hour':(now.hour/2)*2,
             '3_hour':(now.hour/3)*3,
             '5_hour':(now.hour/5)*5,
-            'minute':now.minute
+            'minute':now.minute,
+            '0month': '%0.2d' % now.month,
+            '0day': '%0.2d' % now.day,
+            '0hour': '%0.2d' % now.hour,
+
         }
 
     def init_log_fh(self):
@@ -102,7 +110,7 @@ class Logger(object):
         self.action_log_fh = codecs.open(self.action_log_file,'a','utf8',buffering=self.action_log_buffering)
 
     def flush(self):
-        self.action_log_fh.flush()
+        self.action_log_fh and self.action_log_fh.flush()
 
 
     def arr_lambda_by_name(self,items,mod):
@@ -151,10 +159,11 @@ class Logger(object):
         return u'"' + u',"'.join(map(self.clear_one_ceil,data))
 
     def writerow(self,data):
-        self.action_log_fh.write(u'\n'+self.clear_one_line(data)+',".')
+        self.action_log_fh.write(u'\n'+self.clear_one_line(data))
 
     def store_row(self,data):
-        self.init_log_fh()
+        if self.splitting_blocks:
+            self.init_log_fh()
         self.writerow(data)
 
     def arr_funcs(self,items,*args,**kwargs):
@@ -182,33 +191,41 @@ class Logger(object):
         self.alog(*args,**kwargs)
 
     def alog(self,search,params,stack=1):
+        if not self.splitting_blocks and not self.aindex_stack:
+            #init for new blocks only for a new file
+            self.init_log_fh()
         data = self.tlog(params,stack)
         self.aindex_stack.append([data[0],search])
 
 
     def clog(self,search,params,stack=1):
-        remove_length = 0
+        remove_length = len(self.aindex_stack)
         finded = False
         for item in reversed(self.aindex_stack):
-            remove_length -= 1
             if item[1] == search:
                 finded = True
                 break
+            remove_length -= 1
         if finded:
-            self.tlog(params,stack)
             del self.aindex_stack[remove_length:]
+            self.tlog(params,stack)
+            self.aindex_stack.pop()
         else:
             #convert to simple log
             self.tlog([params[0].split('_')[1]] + params[1:],stack)
             
         if self.aindex_empty_level and len(self.aindex_stack) < self.aindex_empty_level:
             self.aindex_empty_level = None
+        
+    
+    def get_record_time(self):
+        return datetime.now().isoformat()
 
     def tlog(self,params,stack=1):
         '''
             Write a single line
         '''
-        record_key = datetime.now().isoformat()+';'+str(randint(0,100))
+        record_key = self.get_record_time()+';'+str(randint(0,100))
         if self.aindex_empty_level:
             return [record_key]
         arr_row = [record_key,self.get_append_time()] +\
@@ -216,3 +233,13 @@ class Logger(object):
                 self.unpack_params(params)
         self.store_row(arr_row)
         return arr_row
+
+
+class LineLogger(Logger):
+    def tlog(self,params,stack=1):
+        record_key = self.get_record_time()+';'+str(randint(0,100))
+        arr_row = [record_key,] +\
+                self.arr_funcs(self.func_fields,sys._getframe(stack),self) +\
+                self.unpack_params(params)
+        self.store_row(arr_row)
+        return arr_row
